import {PluginItem} from '@babel/core';
import {ProxyConfigMap, ProxyConfigArray} from 'webpack-dev-server';
import {Options} from './start';
import getWebpackConfig, {Configuration} from './getWebpackConfig';
import HtmlWebpackPlugin from 'html-webpack-plugin';
import utils from './utils';
import paths from './paths';

export interface RcbConfig {
    host?: string;
    port?: number;
    proxy?: ProxyConfigMap | ProxyConfigArray;
    mock?: MockConfig;
    babelPresets?: PluginItem[] | null;
    babelPlugins?: PluginItem[] | null;
    alias?: {[key: string]: string}; // alias路径
    open?: boolean; // 是否打开浏览器
    webpackPluginConfig?: HtmlWebpackPlugin.Options; // html-webpack-plugin 配置
    webpack?: Configuration // webpack配置
}

export interface MockConfig {
    rootDir: string;
    api: string;
    whiteList: string[];
}

const defaultMockConfig = {
    rootDir: 'mock', // mock文件的根目录
    api: '/api', // 请求的前缀
    whiteList: [] // 放行的接口
};

export function getRcbConfig(): RcbConfig {
    try {
        return utils.requireModule(paths.getResolvePath('rcb.config.js'));
    }
    catch (e) {
        return {};
    }
}

export function getMockConfig(rcb: RcbConfig) {
    try {
        const mockrc = require(paths.getResolvePath('.mockrc.json'));

        return Object.assign({}, defaultMockConfig, rcb.mock, mockrc);
    }
    catch (e) {
        return Object.assign({}, defaultMockConfig);
    }
}

function getBabelConfig() {
    // TODO
    return {};
}

export default function getExtraConfig(options?: Options) {
    const rcb = getRcbConfig();
    const mockConfig = getMockConfig(rcb);
    rcb.mock = mockConfig;
    return {
        webpack: getWebpackConfig(rcb, options),
        babel: getBabelConfig(),
        rcb
    };
}
