import fs from 'fs';
import path from 'path';
import utils from './utils';
import webpack from 'webpack';

function getPath(fileName: string) {
    const basePath = __dirname;

    return path.resolve(basePath, fileName);
}

function getLoader(filePath: string, ...args: any[]) {
    const getLoaderFn = utils.requireModule(filePath);

    if (utils.isFunction(getLoaderFn)) {
        return getLoaderFn(...args);
    }

    return null;
}

export default function getLoaders(...args: any[]) {
    const fileTypeRegex = /\.js$/;
    const configPath = getPath('getLoaderConfig/');
    const loaderFiles = fs.readdirSync(configPath);
    let loaders: webpack.Module[] = [];

    loaderFiles.forEach(function (loaderFile) {
        // 判断文件类型
        if (fileTypeRegex.test(loaderFile)) {
            const loaderConfig = getLoader(configPath + '/' + loaderFile, ...args);

            if (loaderConfig) {
                loaders.push(loaderConfig);
            }
        }
    });

    return loaders;
};
